package de.twomartens.templateservice.actuator;

import de.twomartens.templateservice.entity.Greeting;
import de.twomartens.templateservice.interceptors.RequestTypeInterceptor;
import de.twomartens.templateservice.interceptors.TraceIdInterceptor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.actuate.health.HealthIndicator;
import org.springframework.boot.autoconfigure.web.ServerProperties;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import java.util.Optional;

/**
 * A Healtcheck which checks if the rest services are working.
 *
 * If you have a complex service behind grpc, you should think about an easy greeting or echo service, which only tests
 * the network/service stack and not the full application.
 *
 * The healthcheck will be called by kubernetes to check if the container/pod shuold be in loadbalancing. It is possible
 * to have as much healthchecks as you loke.
 *
 * There should be a healtcheck which is ok not before all data is loaded.
 */
@Slf4j
@Component
public class RestHealthCheck extends AbstractHealthCheck implements HealthIndicator {

  private final RestTemplate restTemplate;

  public RestHealthCheck(ServerProperties serverProperties,
      TraceIdInterceptor traceIdInterceptor,
      RequestTypeInterceptor requestTypeInterceptor,
      RestTemplate restTemplate) {
    super(Optional.ofNullable(serverProperties.getPort()).orElse(8080),
        traceIdInterceptor, requestTypeInterceptor);
    this.restTemplate = restTemplate;
  }

  @Override
  boolean isEndpointAvailable() {
    try {
      ResponseEntity<Greeting> result = restTemplate.getForEntity(getEndpoint(), Greeting.class);
      Greeting body = result.getBody();
      if (body == null) {
        return false;
      }
      return !body.getMessage().isEmpty();
    } catch (RestClientException e) {
      return false;
    }
  }

  @Override
  String getMethodName() {
    return "infodb-rest";
  }

  String mkEndpoint() {
    return String.format("http://%s:%d/greeting?name=RestHealthCheck", getHost(), getPort());
  }
}
